@@ -43,8 +43,11 @@
 
 #define GLADE_FILE GBF_GLADEDIR "/create_dialogs.glade"
 
-#define COLUMN_FILE 0
-#define COLUMN_URI 1
+enum {
+    COLUMN_FILE,
+    COLUMN_URI,
+    N_COLUMNS
+};
 
 static GtkWidget *
 custom_widget_handler (GladeXML *xml,
@@ -147,22 +150,25 @@ setup_groups_treeview (GbfProjectModel *model,
 }
 
 static void
-error_dialog (GtkWindow *parent, const gchar *msg, ...)
+error_dialog (GtkWindow *parent, const gchar *summary, const gchar *msg, ...)
 {
     va_list ap;
     gchar *tmp;
+    gchar *message;
     GtkWidget *dialog;
     
     va_start (ap, msg);
     tmp = g_strdup_vprintf (msg, ap);
     va_end (ap);
     
-    dialog = gtk_message_dialog_new (parent,
-                                     GTK_DIALOG_DESTROY_WITH_PARENT,
-                                     GTK_MESSAGE_ERROR,
-                                     GTK_BUTTONS_OK,
-                                     tmp);
+    message = g_markup_printf_escaped ("<b>%s</b>\n\n%s", summary, tmp);
+    dialog = gtk_message_dialog_new_with_markup (parent,
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 message);
     g_free (tmp);
+    g_free (message);
     
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -187,7 +193,6 @@ entry_changed_cb (GtkEditable *editable, gpointer user_data)
     g_free (text);
 }
 
-
 gchar*
 gbf_project_util_new_group (GbfProjectModel *model,
                             GtkWindow       *parent,
@@ -253,27 +258,23 @@ gbf_project_util_new_group (GbfProjectModel *model,
                 if (data) {
                     gchar *new_group = NULL;
                     
-                    gtk_widget_hide (dialog);
-                    
                     parent_id = g_strdup (data->id);
                     gbf_tree_data_free (data);
-            
+		    
                     new_group = gbf_project_add_group (project, parent_id, name, &err);
                     if (err) {
-                        error_dialog (parent, _("Can't add group: %s"),
+                        error_dialog (parent, _("Can not add group"), "%s",
                                       err->message);
                         g_error_free (err);
-                    }
+                    } else {
+			finished = TRUE;
+		    }
                     g_free (parent_id);
-                    finished = TRUE;
-                    
                 } else {
-                    error_dialog (parent, _("No parent group selected"));
-                
+                    error_dialog (parent, _("Can not add group"),
+				  "%s", _("No parent group selected"));
                 }
-            
                 g_free (name);
-
                 break;
             }
             default:
@@ -281,7 +282,7 @@ gbf_project_util_new_group (GbfProjectModel *model,
                 break;
         }
     }
-        
+    
     /* destroy stuff */
     gtk_widget_destroy (dialog);
     g_object_unref (gui);
@@ -439,24 +440,22 @@ gbf_project_util_new_target (GbfProjectModel *model,
                 }
                 
                 if (data && type) {
-                    gtk_widget_hide (dialog);
-                    
                     group_id = g_strdup (data->id);
                     gbf_tree_data_free (data);
             
                     new_target = gbf_project_add_target (project, group_id, name, type, &err);
                     if (err) {
-                        error_dialog (parent, _("Can't add target: %s"),
+                        error_dialog (parent, _("Can not add target"), "%s",
                                       err->message);
                         g_error_free (err);
-                    }
-                    g_free (group_id);
+                    } else {
+			finished = TRUE;
+		    }
+		    g_free (group_id);
                     g_free (type);
-                    finished = TRUE;
-                    
                 } else {
-                    error_dialog (parent, _("No group selected"));
-                
+                    error_dialog (parent, _("Can not add target"), "%s",
+				  _("No group selected"));
                 }
             
                 g_free (name);
@@ -539,24 +538,6 @@ setup_targets_treeview (GbfProjectModel *model,
     }
 }
 
-static void
-clean_model(GtkTreeModel* model)
-{
-	GtkTreeIter iter;
-	if (!gtk_tree_model_get_iter_first(model, &iter))
-		return;
-	do
-	{
-		gchar* source_file;
-    	gchar* filename;
-    	gtk_tree_model_get (model, &iter, COLUMN_FILE, &filename,
-        	 COLUMN_URI, &source_file, -1);        
-		g_free(filename);
-		g_free(source_file);
-	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter));
-	gtk_list_store_clear(GTK_LIST_STORE(model));
-}
-
 static void
 browse_button_clicked_cb (GtkWidget *widget, gpointer user_data)
 {
@@ -595,37 +576,36 @@ browse_button_clicked_cb (GtkWidget *widget, gpointer user_data)
     
     gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(dialog),
     	uri ? uri : g_object_get_data (G_OBJECT (widget), "root"));
-    gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
+    gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(dialog), TRUE);
     g_free (uri);
 
-	result = gtk_dialog_run(GTK_DIALOG(dialog));
-	switch(result)
+    result = gtk_dialog_run (GTK_DIALOG (dialog));
+    switch (result)
+    {
+	case GTK_RESPONSE_ACCEPT:
 	{
-		case GTK_RESPONSE_ACCEPT:
-		{
-			GSList* uris = gtk_file_chooser_get_uris(GTK_FILE_CHOOSER(dialog));
-			GSList* filenames = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
-			GSList* node_filename = filenames;
-			GSList* node_uri = uris;
-			clean_model(model);
-			while (node_uri != NULL && node_filename != NULL)
-			{
-				GtkTreeIter iter;
-				gchar* uri = node_uri->data;
-				gchar* file = node_filename->data;
-				gtk_list_store_append(GTK_LIST_STORE(model), &iter);
-				gtk_list_store_set(GTK_LIST_STORE(model), &iter, COLUMN_FILE, file, COLUMN_URI, uri, -1);
-				node_uri = g_slist_next(node_uri);
-				node_filename = g_slist_next(node_filename);
-			}
-  			g_slist_free(uris);
-  			g_slist_free(filenames);
-  			break;
-  		}
-  		default:
-  			break;
-  	} 
-  	gtk_widget_destroy (GTK_WIDGET(dialog));
+	    GSList* uris = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER(dialog));
+	    GSList* node_uri = uris;
+
+	    gtk_list_store_clear (GTK_LIST_STORE (model));
+
+	    while (node_uri != NULL)
+	    {
+		GtkTreeIter iter;
+		gchar* uri = node_uri->data;
+		gchar* file = g_path_get_basename (uri);
+		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+		gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_FILE,
+				    file, COLUMN_URI, uri, -1);
+		node_uri = g_slist_next (node_uri);
+	    }
+	    g_slist_free (uris);
+	    break;
+	}
+	default:
+	    break;
+    } 
+    gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 
 static void
@@ -646,32 +626,32 @@ gbf_project_util_add_source (GbfProjectModel *model,
                              const gchar     *default_uri)
 {
 	gchar* uri = g_strdup(default_uri);
-	GSList* uris = g_slist_append(NULL, uri);
-	GSList* new_sources = 
+	GList* uris = g_list_append(NULL, uri);
+	GList* new_sources = 
 		gbf_project_util_add_source_multi(model, parent, default_target, default_group, uris);
 	g_free(uri);
 	
-	if (g_slist_length(new_sources))
+	if (g_list_length(new_sources))
 	{
 		gchar* new_source = g_strdup(new_sources->data);
 		while(new_sources) 
 		{
 			g_free(new_sources->data);
-			new_sources = g_slist_next(new_sources);
+			new_sources = g_list_next(new_sources);
 		}
-		g_slist_free(new_sources);
+		g_list_free(new_sources);
 		return new_source;
 	}
 	else
 		return NULL;
 }
 
-GSList* 
+GList* 
 gbf_project_util_add_source_multi (GbfProjectModel *model,
-				    GtkWindow       *parent,
-				    const gchar     *default_target,
-				    const gchar     *default_group,
-				    GSList     *uris_to_add)
+				   GtkWindow       *parent,
+				   const gchar     *default_target,
+				   const gchar     *default_group,
+				   GList     *uris_to_add)
 {
     GladeXML *gui;
     GtkWidget *dialog, *source_file_tree;
@@ -684,8 +664,8 @@ gbf_project_util_add_source_multi (GbfProjectModel *model,
     GtkListStore* list;
     GtkCellRenderer* renderer;
     GtkTreeViewColumn* column_filename;
-    GSList* new_sources = NULL;
-    GSList* uri_node;
+    GList* new_sources = NULL;
+    GList* uri_node;
     
     g_return_val_if_fail (model != NULL, NULL);
     
@@ -704,41 +684,51 @@ gbf_project_util_add_source_multi (GbfProjectModel *model,
     ok_button = glade_xml_get_widget (gui, "ok_button");
     
     /* Prepare file tree */
-    list = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
-    gtk_tree_view_set_model(GTK_TREE_VIEW(source_file_tree), GTK_TREE_MODEL(list));
-    renderer = gtk_cell_renderer_text_new();
-    column_filename = gtk_tree_view_column_new_with_attributes("Files", renderer, "text", COLUMN_FILE, NULL);
-    gtk_tree_view_column_set_sizing(column_filename, GTK_TREE_VIEW_COLUMN_FIXED);
-    gtk_tree_view_append_column(GTK_TREE_VIEW(source_file_tree), column_filename);
+    list = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
+    gtk_tree_view_set_model (GTK_TREE_VIEW (source_file_tree),
+			     GTK_TREE_MODEL (list));
+    renderer = gtk_cell_renderer_text_new ();
+    column_filename = gtk_tree_view_column_new_with_attributes ("Files",
+							        renderer,
+							        "text",
+							        COLUMN_FILE,
+							        NULL);
+    gtk_tree_view_column_set_sizing (column_filename,
+				     GTK_TREE_VIEW_COLUMN_FIXED);
+    gtk_tree_view_append_column (GTK_TREE_VIEW (source_file_tree),
+				 column_filename);
     
     /* set up dialog */
    uri_node = uris_to_add;
    while (uri_node) 
    {
         GtkTreeIter iter;
-        gchar* filename = gnome_vfs_get_local_path_from_uri(uri_node->data);
+        gchar* filename = g_path_get_basename (uri_node->data);
         if (!filename)
-        	filename = g_strdup(uri_node->data);
-        gtk_list_store_append(list, &iter);
-        gtk_list_store_set(list, &iter, COLUMN_FILE, filename,
-        	COLUMN_URI, g_strdup(uri_node->data), -1);
-    
-    	uri_node = g_slist_next(uri_node);
+        	filename = g_strdup (uri_node->data);
+        gtk_list_store_append (list, &iter);
+        gtk_list_store_set (list, &iter, COLUMN_FILE, filename,
+			    COLUMN_URI, g_strdup(uri_node->data), -1);
+	g_free (filename);
+    	uri_node = g_list_next (uri_node);
     }
-    if (!g_slist_length(uris_to_add))
-   		gtk_widget_set_sensitive (ok_button, FALSE);
-   	else
-   		gtk_widget_set_sensitive (ok_button, TRUE);
+    if (!g_list_length (uris_to_add))
+	gtk_widget_set_sensitive (ok_button, FALSE);
+    else
+	gtk_widget_set_sensitive (ok_button, TRUE);
 
-    g_signal_connect(G_OBJECT(list), "row_changed", G_CALLBACK(on_row_changed), ok_button);
+    g_signal_connect (G_OBJECT(list), "row_changed",
+		      G_CALLBACK(on_row_changed), ok_button);
     
     g_signal_connect (browse_button, "clicked",
-                      (GCallback) browse_button_clicked_cb, source_file_tree);
+                      G_CALLBACK (browse_button_clicked_cb), source_file_tree);
+    
     g_object_get (project, "project-dir", &project_root, NULL);
     g_object_set_data_full (G_OBJECT (browse_button), "root",
                             project_root, g_free);
     
-    setup_targets_treeview (model, targets_view, default_target, default_group);
+    setup_targets_treeview (model, targets_view, default_target,
+			    default_group);
     gtk_widget_show (targets_view);
     
     if (parent) {
@@ -762,50 +752,62 @@ gbf_project_util_add_source_multi (GbfProjectModel *model,
             
                 data = gbf_project_view_find_selected (GBF_PROJECT_VIEW (targets_view),
                                                        GBF_TREE_NODE_TARGET);
-                if (data) {
-                	GtkTreeIter iter;
-                    gtk_widget_hide (dialog);
-                    
-                    target_id = g_strdup (data->id);
-                    gbf_tree_data_free (data);
-                   
-                   	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(list), &iter))
-                   		break;
-                   	do
-                   	{
-                   		GError *err = NULL;
-						gchar* new_source;
-                   		gchar* source_file;
-                   		gchar* filename;
-                   		gtk_tree_model_get (GTK_TREE_MODEL(list), &iter, COLUMN_FILE, &filename,
-                   			COLUMN_URI, &source_file, -1);
-                   		
-                    	new_source = gbf_project_add_source (project, target_id,
-                                                         source_file, &err);
-                    	if (err) {
-                        	error_dialog (parent, _("Can't add source: %s"),
-                            	          err->message);
-                        	g_error_free (err);
-                    	}
-						else
-							new_sources = g_slist_append(new_sources, new_source);
-						
-						g_free(filename);
-                        g_free(source_file);
-                    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(list), &iter));
-                    
+		if (data) {
+		    GtkTreeIter iter;
+		    GString *err_mesg = g_string_new (NULL);
+		    
+		    target_id = g_strdup (data->id);
+		    gbf_tree_data_free (data);
+		    
+		    if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list),
+							&iter))
+			break;
+		    do
+		    {
+			GError *err = NULL;
+			gchar* new_source;
+			gchar* source_file;
+			gtk_tree_model_get (GTK_TREE_MODEL(list), &iter,
+					    COLUMN_URI, &source_file, -1);
+			
+			new_source = gbf_project_add_source (project,
+							     target_id,
+							     source_file,
+							     &err);
+			if (err) {
+			    gchar *str = g_strdup_printf ("%s: %s\n",
+							  source_file,
+							  err->message);
+			    g_string_append (err_mesg, str);
+			    g_error_free (err);
+			    g_free (str);
+			}
+			else
+			    new_sources = g_list_append (new_sources,
+							 new_source);
+			
+			g_free (source_file);
+		    } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(list),
+						       &iter));
+		    
                     g_free (target_id);
-                    finished = TRUE;
-                    
+		    
+		    if (err_mesg->str && strlen (err_mesg->str) > 0) {
+			error_dialog (parent, _("Can not add source files"),
+				      "%s", err_mesg->str);
+		    } else {
+			finished = TRUE;
+		    }
+		    g_string_free (err_mesg, TRUE);
                 } else {
-                    error_dialog (parent, _("No target has been selected"));
-                
+                    error_dialog (parent, _("Can not add source files"),
+				  "%s", _("No target has been selected"));
                 }
                 
                 break;
             }
             default:
-            	clean_model(GTK_TREE_MODEL(list));
+		gtk_list_store_clear (GTK_LIST_STORE (list));
                 finished = TRUE;
                 break;
         }
